import matplotlib.pyplot as plt
from matplotlib.backends.backend_pdf import PdfPages
import pandas as pd
import re
from .bowtie2QCPlot import plot_bars_of_bowtie2_log

def auto_cast(value):
    '''
    cast value to int or float from str
    '''
    return float(value) if '.' in value else int(value)


def parse_bowtie2_log(log_file):
    '''
    parse bowtie2 log of Pair-End mapping
    input : bowtie2 log file path
    output : stats kv pairs
    '''
    stats = {
        'total_reads': 0,
        'overall_alignment_rate': 0.0,
        'concordant_0': {},
        'concordant_1': {},
        'concordant_more': {},
        'discordant_1': {},
        'mate_0': {},
        'mate_1': {},
        'mate_more': {},
    }

    patterns = {
        'total_reads': r'(\d+) reads;',
        'concordant_0':
        r'(\d+)\s+\(([\d.]+)%\)\s+aligned concordantly 0 times',
        'concordant_1':
        r'(\d+)\s+\(([\d.]+)%\)\s+aligned concordantly exactly 1 time',
        'concordant_more':
        r'(\d+)\s+\(([\d.]+)%\)\s+aligned concordantly >1 times',
        'discordant_1': r'(\d+)\s+\(([\d.]+)%\)\s+aligned discordantly 1 time',
        'mate_0': r'(\d+)\s+\(([\d.]+)%\)\s+aligned 0 times',
        'mate_1': r'(\d+)\s+\(([\d.]+)%\)\s+aligned exactly 1 time',
        'mate_more': r'(\d+)\s+\(([\d.]+)%\)\s+aligned >1 times',
        'overall_alignment_rate': r'([\d.]+)% overall alignment rate',
    }

    with open(log_file, encoding='utf-8') as file:
        for line in file:
            for key, pattern in patterns.items():
                match = re.search(pattern, line)
                if not match:
                    continue
                groups = match.groups()
                if len(groups) == 1:
                    stats[key] = auto_cast(groups[0])
                elif len(groups) == 2:
                    stats[key] = {
                        'count': auto_cast(groups[0]),
                        'percent': auto_cast(groups[1])
                    }

    stats['log_file'] = log_file
    return stats


def procress_logs(log_files):
    '''
    procress bowtie logs to combine fig
    '''
    fig, ax = plt.subplots(len(log_files), 3, figsize=(8.27, 11.69))
    for i, log_file in enumerate(log_files):
        data = parse_bowtie2_log(log_file)
        plot_bars_of_bowtie2_log(data, ax[i])
    plt.savefig(f"tmp.png")
    plt.close(fig)



def procress_logs_with_pdf(log_files):
    '''
    procress bowtie logs to combine fig
    '''
    all_data = []
    with PdfPages('bowtie2_alignment_results.pdf') as pdf:
        fig, ax = plt.subplots(len(log_files), 3, figsize=(8.27, 11.69))
        for i, log_file in enumerate(log_files):
            data = parse_bowtie2_log(log_file)
            flat_data = {
            'total_reads': data['total_reads'],
            'overall_alignment_rate': data['overall_alignment_rate'],
            'log_file': data['log_file'],
            }
            for key in ['concordant_0', 'concordant_1', 'concordant_more', 'discordant_1', 'mate_0', 'mate_1', 'mate_more']:
                flat_data[f'{key}_count'] = data[key]['count']
                flat_data[f'{key}_percent'] = data[key]['percent']
            #df = pd.DataFrame([flat_data])
            #print(df)
            all_data.append(flat_data)
            plot_bars_of_bowtie2_log(data, ax[i])
        df = pd.DataFrame(all_data) 
        print(df)

        plt.subplots_adjust(left=0.15,
                    right=0.9,
                    top=0.9,
                    bottom=0.55,
                    hspace=0.6) 
        pdf.savefig(fig)



def plot_bar(ax, labels, sizes, title, **kwargs):
    '''
    plot bar
    '''
    color = kwargs.get('color', '#078282')
    ax.barh(range(len(labels)), sizes, align='center', color=color)
    ax.set_yticks(range(len(labels)))
    ax.set_yticklabels(labels, fontsize=5)
    ax.set_xlabel('Reads', fontsize=5)
    ax.set_title(title, fontsize=5)
    for idx, value in enumerate(sizes):
        ax.text(value + 10, idx, f'{value:.2f}%', va='center', fontsize=5)
    ax.set_xlim(1, 100)
    ax.tick_params(axis='both', labelsize=5)


def plot_bars_of_bowtie2_log(data, ax):
    '''
    plot bars of a bowtie2 pe log
    '''
    labels_sets = [['0', '1', '>1'], ['1', '0'], ['0', '1', '>1']]

    sizes_sets = [[
        data['concordant_0']['percent'], data['concordant_1']['percent'],
        data['concordant_more']['percent']
    ],
                  [
                      data['discordant_1']['percent'],
                      data['concordant_0']['percent'] +
                      data['discordant_1']['percent']
                  ],
                  [
                      data['mate_0']['percent'], data['mate_1']['percent'],
                      data['mate_more']['percent']
                  ]]

    titles = [
        f'{data["log_file"]}\nConcordant N',
        f'{data["log_file"]}\nConcordant=0`s Breakdown\n(Discordant N)',
        f'{data["log_file"]}\nMate-Level N'
    ]

    colors = ['#078282', '#0ababa', '#03fffa']
    offsets = [500000, 500000, 100000]

    for i in range(3):
        plot_bar(ax[i],
                 labels_sets[i],
                 sizes_sets[i],
                 titles[i],
                 offset=offsets[i],
                 color=colors[i])




import pathlib
from setuptools import setup, find_packages
 
HERE = pathlib.Path(__file__).parent.resolve()
 
PACKAGE_NAME = "bowtie2qc"
AUTHOR = "MING HAN YANG"
AUTHOR_EMAIL = "whuang022@gmail.com"
URL = "https://github.com/whuang022ai/bowtie2QC"
DOWNLOAD_URL = "https://pypi.org/project/bowtie2QC/"
 

VERSION = "0.0.0"
DESCRIPTION = "A python bowtie2 QC ploter"
LONG_DESCRIPTION = (HERE / "README.md").read_text(encoding="utf8")
LONG_DESC_TYPE = "text/markdown"
 
requirements = (HERE / "requirements.txt").read_text(encoding="utf8")
INSTALL_REQUIRES = [s.strip() for s in requirements.split("\n")]
 
dev_requirements = (HERE / "dev_requirements.txt").read_text(encoding="utf8")
EXTRAS_REQUIRE = {"dev": [s.strip() for s in dev_requirements.split("\n")]}
 
CLASSIFIERS = [f"Programming Language :: Python :: 3.{str(v)}" for v in range(7, 12)]
PYTHON_REQUIRES = ">=3.11.5"

setup(
    name=PACKAGE_NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type=LONG_DESC_TYPE,
    author=AUTHOR,
    license= "Apache 2.0", 
    license_files = ('LICENSE.txt',),
    author_email=AUTHOR_EMAIL,
    url=URL,
    download_url=DOWNLOAD_URL,
    python_requires=PYTHON_REQUIRES,
    install_requires=INSTALL_REQUIRES,
    extras_require=EXTRAS_REQUIRE,
    packages=find_packages(),
    classifiers=CLASSIFIERS,
)